//CORE VARIABLES
var canvas;
var gl;

//program handle
var shaderProgram;

//to hold our models, etc
var triangle = {}; 
var quad = {};
var guides = {}; //(points for helping)

function init(){
  //initialize canvas and webgl
  canvas = $('#glcanvas')[0]; //first element from jquery selector
  gl = WebGLUtils.setupWebGL( canvas );
  if (!gl) { alert("Unable to initialize WebGL. Your browser may not support it."); return; }


  //basic params
  gl.clearColor(0.0,  0.0,  0.0,  1.0); //background color
  gl.viewport(0, 0, canvas.width, canvas.height); //viewport setup

  //initialize shaders
  shaderProgram = ShaderUtils.initShaders(gl, 'shaders/vertex.vert', 'shaders/fragment.frag'); //load shaders
  if (shaderProgram < 0) { alert('Unable to initialize shaders.'); return; }
  gl.useProgram(shaderProgram); //specify to use the shaders

  //grab handles for later (store in the program object to keep organized)
  shaderProgram.vertexPositionHandle = gl.getAttribLocation(shaderProgram, "aPosition");
  shaderProgram.colorChoiceHandle = gl.getUniformLocation(shaderProgram,"uColorChoice");
  
  /*** MODEL DATA ***/
  triangle.vertices = [
    0.0, 0.5,
    -0.5, -0.5,
    0.5, -0.5
  ];

  //let opengl understand the data
  var vertexFloats = new Float32Array(triangle.vertices);
  triangle.numVertices = triangle.vertices.length/2;
  triangle.vertexPositionBuffer = gl.createBuffer();
  gl.bindBuffer(gl.ARRAY_BUFFER, triangle.vertexPositionBuffer);
  gl.bufferData(gl.ARRAY_BUFFER, vertexFloats, gl.STATIC_DRAW);

  quad.vertices = [
    -0.5, 0.5,
    -0.5, -0.5,
    0.5, -0.5, 
    0.5, 0.5
  ];
  quad.indices = [
    0,1,2,
    3,0,2
  ];

  vertexFloats = new Float32Array(quad.vertices);
  quad.numVertices = quad.vertices.length/2;
  quad.vertexPositionBuffer = gl.createBuffer();
  gl.bindBuffer(gl.ARRAY_BUFFER, quad.vertexPositionBuffer);
  gl.bufferData(gl.ARRAY_BUFFER, vertexFloats, gl.STATIC_DRAW);

  //indexed...
  var indUints = new Uint16Array(quad.indices);
  quad.numIndices = quad.indices.length;
  quad.vertexIndexBuffer = gl.createBuffer();
  gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, quad.vertexIndexBuffer);
  gl.bufferData(gl.ELEMENT_ARRAY_BUFFER, indUints, gl.STATIC_DRAW);


  /*** GUIDES ***/
  var verts = [0,0,  -1,-1,  -1,1,   1,-1,   1,1,   -0.5,0,   0.5,0];
  var vertFloats = new Float32Array(verts);
  guides.numVertices = verts.length/2;
  guides.vertexPositionBuffer = gl.createBuffer();
  gl.bindBuffer(gl.ARRAY_BUFFER, guides.vertexPositionBuffer);
  gl.bufferData(gl.ARRAY_BUFFER, vertFloats, gl.STATIC_DRAW);

}

function drawTriangle()
{
  //bind buffers for drawing
  gl.bindBuffer(gl.ARRAY_BUFFER, triangle.vertexPositionBuffer);

  //point in shader variables
  gl.vertexAttribPointer(shaderProgram.vertexPositionHandle, 2, gl.FLOAT, false, 0, 0);
  gl.enableVertexAttribArray(shaderProgram.vertexPositionHandle);
  gl.uniform1i(shaderProgram.colorChoiceHandle,0);

  //draw the model!
  gl.drawArrays(gl.TRIANGLES, 0, triangle.numVertices);
}

function drawQuad()
{
  //bind the quad's buffers for drawing
  gl.bindBuffer(gl.ARRAY_BUFFER, quad.vertexPositionBuffer);
  gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, quad.vertexIndexBuffer);

  //point in shader variables
  gl.vertexAttribPointer(shaderProgram.vertexPositionHandle, 2, gl.FLOAT, false, 0, 0);
  gl.enableVertexAttribArray(shaderProgram.vertexPositionHandle);
  gl.uniform1i(shaderProgram.colorChoiceHandle,0);

  //draw the model!
  gl.drawElements(gl.TRIANGLES, quad.numIndices, gl.UNSIGNED_SHORT, 0);
}


function drawGuides() {
  gl.bindBuffer(gl.ARRAY_BUFFER, guides.vertexPositionBuffer);
  gl.vertexAttribPointer(shaderProgram.vertexPositionHandle, 2, gl.FLOAT, false, 0, 0);
  gl.enableVertexAttribArray(shaderProgram.vertexPositionHandle);
  gl.uniform1i(shaderProgram.colorChoiceHandle,1);
  gl.drawArrays(gl.POINTS,0,guides.numVertices); //actually draw the points
}


function render(){
  gl.clear(gl.COLOR_BUFFER_BIT);

  //drawGuides();

  drawTriangle();
  //drawQuad();
}


$(document).ready(function(){
  init(); //initialize everything
  render(); //start drawing
});

